#!/usr/bin/env python
# Copyright: This document has been placed in the public domain.

"""
Taylor diagram (Taylor, 2001) test implementation.

http://www-pcmdi.llnl.gov/about/staff/Taylor/CV/Taylor_diagram_primer.htm
"""

__version__ = "Time-stamp: <2012-02-17 20:59:35 ycopin>"
__author__ = "Yannick Copin <yannick.copin@laposte.net>"

import numpy as NP
import matplotlib.pyplot as PLT

class TaylorDiagram(object):
    """Taylor diagram: plot model standard deviation and correlation
    to reference (data) sample in a single-quadrant polar plot, with
    r=stddev and theta=arccos(correlation).
    """

    def __init__(self, refstd, fig=None, rect=111, label='_'):
        """Set up Taylor diagram axes, i.e. single quadrant polar
        plot, using mpl_toolkits.axisartist.floating_axes. refstd is
        the reference standard deviation to be compared to.
        """

        from matplotlib.projections import PolarAxes
        import mpl_toolkits.axisartist.floating_axes as FA
        import mpl_toolkits.axisartist.grid_finder as GF

        #self.refstd = refstd            # Reference standard deviation
        self.refstd = 1            # Reference standard deviation

        tr = PolarAxes.PolarTransform()

        # Correlation labels
        rlocs = NP.concatenate((NP.arange(10)/10.,[0.95,0.99]))
        rlocs2 = NP.arange(8)/4.
        #print(rlocs)
        print(NP.arange(8))
        print(rlocs2)
        tlocs = NP.arccos(rlocs)        # Conversion to polar angles
        #tlocs2 = NP.arccos(rlocs2)
        tlocs2 = NP.arange(8)/4.
        print(tlocs)
        print(tlocs2)
        gl1 = GF.FixedLocator(tlocs)    # Positions
        tf1 = GF.DictFormatter(dict(zip(tlocs, map(str,rlocs))))
        gl2 = GF.FixedLocator(tlocs2)  
        tf2 = GF.DictFormatter(dict(zip(tlocs2, map(str,rlocs2))))


        # Standard deviation axis extent
        self.smin = 0.
        self.smax = 1.75*self.refstd

        ghelper = FA.GridHelperCurveLinear(tr,
                                           extremes=(0,NP.pi/2, # 1st quadrant
                                                     self.smin,self.smax),
                                           grid_locator1=gl1,
                                           tick_formatter1=tf1,
                                           )


        ghelper2 = FA.GridHelperCurveLinear(tr,
                                           extremes=(0,NP.pi/2, # 1st quadrant
                                                     self.smin,self.smax),
                                           grid_locator1=gl1,
                                           tick_formatter1=tf1,
                                           grid_locator2=gl2,
                                           tick_formatter2=tf2,
                                           )




        if fig is None:
            fig = PLT.figure()

        #plm ax = FA.FloatingSubplot(fig, rect, grid_helper=ghelper)
        ax = FA.FloatingSubplot(fig, rect, grid_helper=ghelper2)
        fig.add_subplot(ax)

        # Adjust axes
        ax.axis["top"].set_axis_direction("bottom")  # "Angle axis"
        ax.axis["top"].toggle(ticklabels=True, label=True)
        ax.axis["top"].major_ticklabels.set_axis_direction("top")
        ax.axis["top"].label.set_axis_direction("top")
        ax.axis["top"].label.set_text("Correlation")

        ax.axis["left"].set_axis_direction("bottom") # "X axis"
        ax.axis["left"].label.set_text("Normalized Standard Deviation")

        ax.axis["right"].set_axis_direction("top")   # "Y axis"
        ax.axis["right"].toggle(ticklabels=True)
        ax.axis["right"].major_ticklabels.set_axis_direction("left")
        ax.axis["right"].label.set_text("Normalized Standard Deviation")

        ax.axis["bottom"].set_visible(False)         # Useless

        # Contours along standard deviations
        #plm ax.grid(False)
        ax.grid(True,linestyle='--')

        self._ax = ax                   # Graphical axes
        self.ax = ax.get_aux_axes(tr)   # Polar coordinates

        # Add reference point and stddev contour
        print "Reference std:", self.refstd
        #plm l, = self.ax.plot([0], self.refstd, 'k*',
        #plm                   ls='', ms=10, label=label)
        label='OBS'
        l, = self.ax.plot([0], self.refstd, 'ko',
                          ls='', ms=10, label=label)
        t = NP.linspace(0, NP.pi/2)
        r = NP.zeros_like(t) + self.refstd
        #plm self.ax.plot(t,r, 'k--', label='_')

        # Collect sample points for latter use (e.g. legend)
        self.samplePoints = [l]

    def add_sample(self, stddev, corrcoef, *args, **kwargs):
        """Add sample (stddev,corrcoeff) to the Taylor diagram. args
        and kwargs are directly propagated to the Figure.plot
        command."""

        l, = self.ax.plot(NP.arccos(corrcoef), stddev,
                          *args, **kwargs) # (theta,radius)
        self.samplePoints.append(l)

        return l

    def add_contours(self, levels=4, **kwargs):
        """Add constant centered RMS difference contours."""

        rs,ts = NP.meshgrid(NP.linspace(self.smin,self.smax),
                            NP.linspace(0,NP.pi/2))
        # Compute centered RMS difference
        rms = NP.sqrt(self.refstd**2 + rs**2 - 2*self.refstd*rs*NP.cos(ts))

        contours = self.ax.contour(ts, rs, rms, levels, **kwargs)

        return contours


if __name__=='__main__':

    # Reference dataset
    #x = NP.linspace(0,4*NP.pi,100)
    #data = NP.sin(x)
    #refstd = data.std(ddof=1)           # Reference standard deviation

    # Models
    #m1 = data + 0.2*NP.random.randn(len(x))    # Model 1
    #m2 = 0.8*data + .1*NP.random.randn(len(x)) # Model 2
    #m3 = NP.sin(x-NP.pi/10)                    # Model 3

    # Compute stddev and correlation coefficient of models
    #samples = NP.array([ [m.std(ddof=1), NP.corrcoef(data, m)[0,1]]
    #                     for m in (m1,m2,m3)])

    #fig = PLT.figure(figsize=(10,4))

    #ax1 = fig.add_subplot(1,2,1, xlabel='X', ylabel='Y')
    # Taylor diagram
    #dia = TaylorDiagram(refstd, fig=fig, rect=122, label="Reference")

    xp=['1','2','3','4','5','6','7','8','9','10','11','12','13','14','15','16','17','18','19','20','21']
    xp=['GDPS4','D95','ISBAES','CHTESSEL','LMDZ','NOAH','CROCUS','EBA','JULES','CLM4','11','12','13','14','15','16','17','18','19','20','21']
    #colors=['blue','red','cyan','green','orange','indigo','magenta','yellow','black','brown','purple','palegreen' ]

    serie='ALL'
    samples=NP.array([ [ 1.00799417,0.930621326],                         # out.SnowT_cmc_gdps_v02            
                       [0.890084624,0.966879725],                         # out.SnowT_cnrm_d95_v0             
                       [ 1.05014086,0.950379908],                         # out.SnowT_cnrm_ies_v8             
                       [ 1.01942992,0.978896141],                         # out.SnowT_ecmwf_chtessel_v01_sn_z0
                       [0.889409244,0.969795883],                         # out.SnowT_lgge_lmdz_v1            
                       [0.975187480,0.925700724],                         # out.SnowT_ncep_noah_v3            
                       #[ 1.05412853,0.955837190],                         # out.SnowT_cnrm_cro_new            
                       #[0.927484393,0.969937205],                         # out.SnowT_cnrm_eba_new            
                       #[ 1.20721328,0.973771930],                         # out.SnowT_ecmwf_chtessel_new      
                       #[0.895476341,0.972798884],                         # out.SnowT_larc_clm4_new           
                       #[ 1.18500805,0.936257243],                         # out.SnowT_mo_jules_new            
                       [ 1.05412853,0.955837190],                         # out.SnowT_cnrm_cro_v8             
                       [0.848216295,0.960280716],                         # out.SnowT_cnrm_eba_v0             
                       #[0.790529430,0.888769984],                         # out.SnowT_ecmwf_chtessel_v01      
                       #[0.894597113,0.950327516],                         # out.SnowT_larc_clm4_v1            
                       [ 1.22563124,0.936282694],                         # out.SnowT_mo_jules_v13            
                       #[0.968145728,0.973095596],                         # out.SnowT_cnrm_d95_new            
                       #[ 1.02473569,0.953877270],                         # out.SnowT_cnrm_ies_new            
                       #[0.959507883,0.974980712],                         # out.SnowT_ecmwf_chtessel_v01_sn   
                       [0.885070682,0.972111762]  ]) #,                         # out.SnowT_larc_clm4_v2            
                       #[0.972695947,0.921349764]  ])                      # out.SnowT_ncep_noah_new           







    serie='DAY'
    tlabel='(a)'
    samples=NP.array([    [  1.44644141,0.870792687],            # out.SnowT_cmc_gdps_v02            
                         [ 0.955398500,0.906596363],            # out.SnowT_cnrm_d95_v0             
                         [  1.07700300,0.891818762],            # out.SnowT_cnrm_ies_v8             
                         [  1.13503301,0.907311440],            # out.SnowT_ecmwf_chtessel_v01_sn_z0
                         [  1.04697609,0.933073163],            # out.SnowT_lgge_lmdz_v1            
                         [ 0.903253019,0.867113352],            # out.SnowT_ncep_noah_v3            
                         #[ 0.950919867,0.892167747],             # out.SnowT_cnrm_cro_new           
                         #[ 0.934291899,0.903217673],             # out.SnowT_cnrm_eba_new           
                         #[  1.05042076,0.905650139],             # out.SnowT_ecmwf_chtessel_new     
                         #[  1.02847028,0.923704028],             # out.SnowT_larc_clm4_new          
                         #[ 0.892629147,0.839797139],             # out.SnowT_mo_jules_new           
                         [ 0.950919867,0.892167747],            # out.SnowT_cnrm_cro_v8             
                         [ 0.967561781,0.901265919],            # out.SnowT_cnrm_eba_v0             
                         #[  1.09454417,0.816538155],            # out.SnowT_ecmwf_chtessel_v01     
                         #[  1.16306031,0.936644435],             # out.SnowT_larc_clm4_v1           
                         [ 0.877002060,0.880718529],            # out.SnowT_mo_jules_v13            
                         #[ 0.946252406,0.902406573],             # out.SnowT_cnrm_d95_new           
                         #[  1.10068691,0.896762073],             # out.SnowT_cnrm_ies_new           
                         #[  1.07923031,0.912361801],             # out.SnowT_ecmwf_chtessel_v01_sn  
                         [  1.00431061,0.936870098]  ]) #,                  # out.SnowT_larc_clm4_v2      
                         #[ 0.892303348,0.849767923],             # out.SnowT_ncep_noah_new          

    serie='NIGHT'
    tlabel='(b)'
    samples=NP.array([ [1.50127077,0.890591979],         # out.SnowT_cmc_gdps_v02            
                      [1.16656184,0.929293156],         # out.SnowT_cnrm_d95_v0             
                      [1.37255383,0.905790389],         # out.SnowT_cnrm_ies_v8             
                      [1.12791705,0.943516076],         # out.SnowT_ecmwf_chtessel_v01_sn_z0
                      [1.21305418,0.936005175],         # out.SnowT_lgge_lmdz_v1            
                      [1.33448720,0.858505785],         # out.SnowT_ncep_noah_v3            
                      #[1.27649379,0.905160725],           # out.SnowT_cnrm_cro_new           
                      #[1.16742623,0.930555403],           # out.SnowT_cnrm_eba_new           
                      #[1.45276272,0.935740530],           # out.SnowT_ecmwf_chtessel_new     
                      #[1.10620081,0.924519897],           # out.SnowT_larc_clm4_new          
                      #[1.36654496,0.868100643],           # out.SnowT_mo_jules_new           
                      [1.27649379,0.905160725],         # out.SnowT_cnrm_cro_v8             
                      [1.16138744,0.918817282],         # out.SnowT_cnrm_eba_v0             
                      #[1.05897391,0.832714617],          # out.SnowT_ecmwf_chtessel_v01     
                      #[1.26918912,0.891469300],           # out.SnowT_larc_clm4_v1           
                      [1.56138241,0.840932667],         # out.SnowT_mo_jules_v13            
                      #[1.14578617,0.934554517],           # out.SnowT_cnrm_d95_new           
                      #[1.35924017,0.910489082],           # out.SnowT_cnrm_ies_new           
                      #[1.15875137,0.938370883],           # out.SnowT_ecmwf_chtessel_v01_sn  
                      [1.08699012,0.910063148]  ]) #,                     # out.SnowT_larc_clm4_v2      
                      #[1.16806507,0.861387730],           # out.SnowT_ncep_noah_new          



    print serie
    print samples




    #plm colors = PLT.matplotlib.cm.jet(NP.linspace(0,1,len(samples)))
    colors = PLT.matplotlib.cm.jet(NP.linspace(0,1,len(samples)))
    #plm: pour avoir les mm couleurs que pour le scatter de Qh
    colors = NP.array(["red","green","blue","yellow","black","orange","purple","brown","cyan","magenta"])
    print colors

    #ax1.plot(x,data,'ko', label='Data')
    #for i,m in enumerate([m1,m2,m3]):
    #    ax1.plot(x,m, c=colors[i], label='Model %d' % (i+1))
    #ax1.legend(numpoints=1, prop=dict(size='small'), loc='best')

    fig = PLT.figure()
    refstd=1
    dia = TaylorDiagram(refstd, fig=fig, rect=111, label="Reference")

    # Add samples to Taylor diagram
    #for i,(stddev,corrcoef) in enumerate(samples):
    #    dia.add_sample(stddev, corrcoef, marker='s', ls='', c=colors[i],
    #                   label="Model %d" % (i+1))

    # Add RMS contours, and label them
    contours = dia.add_contours(colors='0.8')
    PLT.clabel(contours, inline=1, fontsize=9, linestyle='dotted')

    # Add samples to Taylor diagram
    #dia.add_sample(1.2, 0.95, marker='o', ls='', ms=10, c='blue',label="SIM_REF")
    #dia.add_sample(1.2, 0.99, marker='o', ls='', ms=10, c='red',label="SIM_PHY")
    for i,(stddev,corrcoef) in enumerate(samples):
        dia.add_sample(stddev, corrcoef, marker=(5,1), ms='10', ls='', c=colors[i],
                       label=xp[i])

    # Add text to the figure
    PLT.text(-0.32,1.75,tlabel,fontsize=12,fontweight='bold')

    # Add a figure legend
    fig.legend(dia.samplePoints,
               [ p.get_label() for p in dia.samplePoints ],
               numpoints=1, prop=dict(size='small'), loc=(0.755,0.52))

    PLT.savefig(serie+'_taylor.png',dpi=300,bbox_inches='tight') 
    PLT.show()
